"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const logger_factory_1 = require("./logger.factory");
const jsforce = __importStar(require("jsforce"));
const util_1 = require("./util");
const sf_api_shared_1 = require("@shingo/sf-api-shared");
const OMIT_FIELDS = [
    'LastModifiedDate',
    'IsDeleted',
    'LastViewedDate',
    'LastReferencedDate',
    'SystemModstamp',
    'CreatedById',
    'CreatedDate',
    'LastModifiedById',
    'JigsawCompanyId',
    'PhotoUrl',
    'MasterRecordId',
    'IsEmailBounced',
    'OtherAddress',
    'LastCUUpdateDate',
    'Contact_Quality__c',
    'MailingAddress',
    'LastCURequestDate',
    'LastActivityDate',
    'JigsawContactId',
    'password',
    'Account',
    'Facilitator_For__r',
    'id',
    'role',
    'RecordType',
];
class SalesforceService {
    constructor(SF_URL, SF_ENV, SF_USER, SF_PASS, conn = new jsforce.Connection({
        loginUrl: SF_URL,
        instanceUrl: SF_ENV,
    }), log = logger_factory_1.loggerFactory(), auditLog = logger_factory_1.loggerFactory('salesforce-api.audit.log')) {
        this.conn = conn;
        this.log = log;
        this.auditLog = auditLog;
        this.queryRunner = util_1.runQuery(SF_USER, SF_PASS, this.conn);
    }
    query(queryRequest) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                let queryString = `SELECT ${queryRequest.fields.join(',')} FROM ${queryRequest.table}`;
                if (queryRequest.clauses)
                    queryString += ' WHERE ' + queryRequest.clauses;
                this.log.debug('Executing SOQL: %s', queryString);
                return this.queryRunner((conn) => __awaiter(this, void 0, void 0, function* () {
                    const res = yield conn.query(queryString);
                    return new sf_api_shared_1.JSONObject(util_1.deepClean(res, 'attributes'));
                }));
            }
            catch (error) {
                this.log.error('Error in SalesforceService.query(): %j', error);
                throw error;
            }
        });
    }
    retrieve(idRequest) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return this.queryRunner((conn) => __awaiter(this, void 0, void 0, function* () {
                    const res = yield conn.sobject(idRequest.object).retrieve(idRequest.ids);
                    return new sf_api_shared_1.JSONObject(util_1.deepClean(res, 'attributes'));
                }));
            }
            catch (error) {
                this.log.error('Error in SalesforceService.retrieve(): %j', error);
                throw error;
            }
        });
    }
    create(recordsRequest) {
        return __awaiter(this, void 0, void 0, function* () {
            const records = util_1.getRecords(recordsRequest, OMIT_FIELDS);
            try {
                return this.queryRunner(conn => {
                    this.auditLog.info('Creating new records: %j', records);
                    return conn
                        .sobject(recordsRequest.object)
                        .create(records)
                        .then(o => new sf_api_shared_1.JSONObject(o));
                });
            }
            catch (error) {
                this.log.error('Error in SalesforceService.create(): %j', error);
                throw error;
            }
        });
    }
    update(recordsRequest) {
        return __awaiter(this, void 0, void 0, function* () {
            const records = util_1.getRecords(recordsRequest, OMIT_FIELDS);
            try {
                return this.queryRunner(conn => {
                    this.auditLog.info('Updating records: %j', records);
                    return conn
                        .sobject(recordsRequest.object)
                        .update(records)
                        .then(o => new sf_api_shared_1.JSONObject(o));
                });
            }
            catch (error) {
                this.log.error('Error in SalesforceService.update(): %j', error);
                throw error;
            }
        });
    }
    delete(idRequest) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return this.queryRunner(conn => {
                    this.auditLog.info('Deleting records: %j', idRequest.ids);
                    return conn
                        .sobject(idRequest.object)
                        .delete(idRequest.ids)
                        .then(o => new sf_api_shared_1.JSONObject(o));
                });
            }
            catch (error) {
                this.log.error('Error in SalesforceService.delete(): %j', error);
                throw error;
            }
        });
    }
    upsert(request) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const records = util_1.getRecords(request);
                return this.queryRunner(conn => {
                    this.auditLog.info('Upsert records for %s: %j', request.object, records);
                    return conn
                        .sobject(request.object)
                        .upsert(records, request.extId)
                        .then(o => new sf_api_shared_1.JSONObject(o));
                });
            }
            catch (error) {
                this.log.error('Error in SalesforceService.upsert(): %j', error);
                throw error;
            }
        });
    }
    describe(object) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return this.queryRunner(conn => conn
                    .sobject(object)
                    .describe()
                    .then(o => new sf_api_shared_1.JSONObject(o)));
            }
            catch (error) {
                this.log.error('Error in SalesforceService.describe(): %j', error);
                throw error;
            }
        });
    }
    search(searchRequest) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return this.queryRunner((conn) => __awaiter(this, void 0, void 0, function* () {
                    // FIXME: jsforce typings are incorrect
                    const res = yield conn.search(`FIND ${searchRequest.search} IN ALL FIELDS RETURNING ${searchRequest.retrieve}`);
                    return new sf_api_shared_1.JSONObject(util_1.deepClean(res, 'attributes'));
                }));
            }
            catch (error) {
                this.log.error('Error in SalesforceService.search(): %j', error);
                throw error;
            }
        });
    }
}
exports.SalesforceService = SalesforceService;
//# sourceMappingURL=salesforce.service.js.map