"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const grpc_1 = require("grpc");
const lodash_1 = __importDefault(require("lodash"));
/**
 * Authenticates a connection to salesforce and calls a function with the logged in connection
 * @param username Salesforce username
 * @param password Salesforce password
 * @param connection JSForce connection object
 * @param fn The function to run
 */
exports.runQuery = (username, password, connection) => (fn) => connection
    .login(username, password)
    .then(() => fn(connection))
    .then(res => {
    connection.logout();
    return res;
});
class SError extends Error {
    constructor(error, status) {
        super();
        this.message = error.message;
        this.name = error.name || 'Error';
        this.metadata = new grpc_1.Metadata();
        const stringErr = JSON.stringify(error, (_key, value) => {
            if (value instanceof Error) {
                const err = {};
                Object.getOwnPropertyNames(error).forEach(k => {
                    err[k] = value[k];
                });
                return err;
            }
            return value;
        });
        this.metadata.add('error-bin', Buffer.from(stringErr));
        this.code = status || grpc_1.status.INTERNAL;
    }
}
exports.SError = SError;
/**
 * Handles a GRPC unary request
 * @param logger Logger instance
 * @param name Function name for logging
 * @param fn Handler function implementation
 */
exports.handleUnary = (logger) => (name, fn) => (call, cb) => {
    fn(call.request)
        .then(record => cb(null, record))
        .catch(error => {
        logger.error(`Error in ${name}(): %j`, error);
        cb(new SError(error), null);
    });
};
exports.getRecords = (req, omit = []) => req.records.map(record => {
    const val = record.value;
    return typeof val === 'object' ? lodash_1.default.omit(val, omit) : val;
});
const removeKey = (o) => (key) => {
    if (lodash_1.default.isObject(o)) {
        Object.keys(o).forEach(k => {
            if (k === key) {
                delete o[k];
            }
            else if (lodash_1.default.isObject(o[k])) {
                // for very deep objects this could cause the stack to overflow
                // but this did not occur ever when using the deep-clean library
                removeKey(o[k])(key);
            }
        });
    }
};
exports.deepClean = (o, ...keys) => {
    const newObj = lodash_1.default.cloneDeep(o);
    keys.forEach(removeKey(newObj));
    return newObj;
};
//# sourceMappingURL=util.js.map